@@ -29,7 +29,7 @@ if (!defined('Adminer\DRIVER')) {
 					($server . $username . $password != "" ? $password : ini_get("mysqli.default_pw")),
 					null,
 					(is_numeric($port) ? intval($port) : ini_get("mysqli.default_port")),
-					(is_numeric($port) ? $port : null),
+					(is_numeric($port) ? null : $port),
 					($ssl ? ($ssl['verify'] !== false ? 2048 : 64) : 0) // 2048 - MYSQLI_CLIENT_SSL, 64 - MYSQLI_CLIENT_SSL_DONT_VERIFY_SERVER_CERT (not available before PHP 5.6.16)
 				);
 				$this->options(MYSQLI_OPT_LOCAL_INFILE, false);
@@ -258,6 +258,9 @@ if (!defined('Adminer\DRIVER')) {
 				$this->types[lang('Numbers')]["vector"] = 16383;
 				$this->insertFunctions['vector'] = 'string_to_vector';
 			}
+			if (min_version(5.1, '', $connection)) {
+				$this->partitionBy = array("HASH", "LINEAR HASH", "KEY", "LINEAR KEY", "RANGE", "LIST");
+			}
 			if (min_version(5.7, 10.2, $connection)) {
 				$this->generated = array("STORED", "VIRTUAL");
 			}
@@ -335,6 +338,17 @@ if (!defined('Adminer\DRIVER')) {
 			}
 		}
 
+		function partitionsInfo(string $table): array {
+			$from = "FROM information_schema.PARTITIONS WHERE TABLE_SCHEMA = " . q(DB) . " AND TABLE_NAME = " . q($table);
+			$result = connection()->query("SELECT PARTITION_METHOD, PARTITION_EXPRESSION, PARTITION_ORDINAL_POSITION $from ORDER BY PARTITION_ORDINAL_POSITION DESC LIMIT 1");
+			$return = array();
+			list($return["partition_by"], $return["partition"], $return["partitions"]) = $result->fetch_row();
+			$partitions = get_key_vals("SELECT PARTITION_NAME, PARTITION_DESCRIPTION $from AND PARTITION_NAME != '' ORDER BY PARTITION_ORDINAL_POSITION");
+			$return["partition_names"] = array_keys($partitions);
+			$return["partition_values"] = array_values($partitions);
+			return $return;
+		}
+
 		function hasCStyleEscapes(): bool {
 			static $c_style;
 			if ($c_style === null) {
@@ -353,6 +367,10 @@ if (!defined('Adminer\DRIVER')) {
 			}
 			return $return;
 		}
+
+		function indexAlgorithms(array $tableStatus): array {
+			return (preg_match('~^(MEMORY|NDB)$~', $tableStatus["Engine"]) ? array("HASH", "BTREE") : array());
+		}
 	}
 
 
@@ -544,6 +562,7 @@ if (!defined('Adminer\DRIVER')) {
 			$return[$name]["columns"][] = $row["Column_name"];
 			$return[$name]["lengths"][] = ($row["Index_type"] == "SPATIAL" ? null : $row["Sub_part"]);
 			$return[$name]["descs"][] = null;
+			$return[$name]["algorithm"] = $row["Index_type"];
 		}
 		return $return;
 	}
@@ -677,9 +696,10 @@ if (!defined('Adminer\DRIVER')) {
 	* @param list<array{string, list<string>, string}> $fields of [$orig, $process_field, $after]
 	* @param string[] $foreign
 	* @param numeric-string $auto_increment
+	* @param ?Partitions $partitioning null means remove partitioning
 	* @return Result|bool
 	*/
-	function alter_table(string $table, string $name, array $fields, array $foreign, ?string $comment, string $engine, string $collation, string $auto_increment, string $partitioning) {
+	function alter_table(string $table, string $name, array $fields, array $foreign, ?string $comment, string $engine, string $collation, string $auto_increment, ?array $partitioning) {
 		$alter = array();
 		foreach ($fields as $field) {
 			if ($field[1]) {
@@ -700,8 +720,28 @@ if (!defined('Adminer\DRIVER')) {
 			. ($collation ? " COLLATE " . q($collation) : "")
 			. ($auto_increment != "" ? " AUTO_INCREMENT=$auto_increment" : "")
 		;
+
+		if ($partitioning) {
+			$partitions = array();
+			if ($partitioning["partition_by"] == 'RANGE' || $partitioning["partition_by"] == 'LIST') {
+				foreach ($partitioning["partition_names"] as $key => $val) {
+					$value = $partitioning["partition_values"][$key];
+					$partitions[] = "\n  PARTITION " . idf_escape($val) . " VALUES " . ($partitioning["partition_by"] == 'RANGE' ? "LESS THAN" : "IN") . ($value != "" ? " ($value)" : " MAXVALUE"); //! SQL injection
+				}
+			}
+			// $partitioning["partition"] can be expression, not only column
+			$status .= "\nPARTITION BY $partitioning[partition_by]($partitioning[partition])";
+			if ($partitions) {
+				$status .= " (" . implode(",", $partitions) . "\n)";
+			} elseif ($partitioning["partitions"]) {
+				$status .= " PARTITIONS " . (+$partitioning["partitions"]);
+			}
+		} elseif ($partitioning === null) {
+			$status .= "\nREMOVE PARTITIONING";
+		}
+
 		if ($table == "") {
-			return queries("CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)$status$partitioning");
+			return queries("CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)$status");
 		}
 		if ($table != $name) {
 			$alter[] = "RENAME TO " . table($name);
@@ -709,12 +749,12 @@ if (!defined('Adminer\DRIVER')) {
 		if ($status) {
 			$alter[] = ltrim($status);
 		}
-		return ($alter || $partitioning ? queries("ALTER TABLE " . table($table) . "\n" . implode(",\n", $alter) . $partitioning) : true);
+		return ($alter ? queries("ALTER TABLE " . table($table) . "\n" . implode(",\n", $alter)) : true);
 	}
 
 	/** Run commands to alter indexes
 	* @param string $table escaped table name
-	* @param list<array{string, string, 'DROP'|list<string>}> $alter of ["index type", "name", ["column definition", ...]] or ["index type", "name", "DROP"]
+	* @param list<array{string, string, 'DROP'|list<string>, 3?: string, 4?: string}> $alter of ["index type", "name", ["column definition", ...], "algorithm", "condition"] or ["index type", "name", "DROP"]
 	* @return Result|bool
 	*/
 	function alter_indexes(string $table, $alter) {
@@ -1012,10 +1052,18 @@ if (!defined('Adminer\DRIVER')) {
 	}
 
 	/** Check whether a feature is supported
-	* @param literal-string $feature "check|comment|copy|database|descidx|drop_col|dump|event|indexes|kill|materializedview|partitioning|privileges|procedure|processlist|routine|scheme|sequence|status|table|trigger|type|variables|view|view_trigger"
+	* @param literal-string $feature check|comment|columns|copy|database|descidx|drop_col|dump|event|indexes|kill|materializedview
+	* |move_col|privileges|procedure|processlist|routine|scheme|sequence|sql|status|table|trigger|type|variables|view|view_trigger
 	*/
 	function support(string $feature): bool {
-		return !preg_match("~scheme|sequence|type|view_trigger|materializedview" . (min_version(8) ? "" : "|descidx" . (min_version(5.1) ? "" : "|event|partitioning")) . (min_version('8.0.16', '10.2.1') ? "" : "|check") . "~", $feature);
+		return preg_match(
+			'~^(comment|columns|copy|database|drop_col|dump|indexes|kill|privileges|move_col|procedure|processlist|routine|sql|status|table|trigger|variables|view'
+				. (min_version(5.1) ? '|event' : '')
+				. (min_version(8) ? '|descidx' : '')
+				. (min_version('8.0.16', '10.2.1') ? '|check' : '')
+				. ')$~',
+			$feature
+		);
 	}
 
 	/** Kill a process
