@@ -127,8 +127,9 @@ if (isset($_GET["pgsql"])) {
 				$return = new \stdClass;
 				$return->orgtable = pg_field_table($this->result, $column);
 				$return->name = pg_field_name($this->result, $column);
-				$return->type = pg_field_type($this->result, $column); //! map to MySQL numbers
-				$return->charsetnr = ($return->type == "bytea" ? 63 : 0); // 63 - binary
+				$type = pg_field_type($this->result, $column);
+				$return->type = (preg_match(number_type(), $type) ? 0 : 15);
+				$return->charsetnr = ($type == "bytea" ? 63 : 0); // 63 - binary
 				return $return;
 			}
 
@@ -203,10 +204,12 @@ if (isset($_GET["pgsql"])) {
 		static $extensions = array("PgSQL", "PDO_PgSQL");
 		static $jush = "pgsql";
 
-		public $operators = array("=", "<", ">", "<=", ">=", "!=", "~", "!~", "LIKE", "LIKE %%", "ILIKE", "ILIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT IN", "IS NOT NULL"); // no "SQL" to avoid CSRF
+		public $operators = array("=", "<", ">", "<=", ">=", "!=", "~", "!~", "LIKE", "LIKE %%", "ILIKE", "ILIKE %%", "IN", "IS NULL", "NOT LIKE", "NOT ILIKE", "NOT IN", "IS NOT NULL"); // no "SQL" to avoid CSRF
 		public $functions = array("char_length", "lower", "round", "to_hex", "to_timestamp", "upper");
 		public $grouping = array("avg", "count", "count distinct", "max", "min", "sum");
 
+		public string $nsOid = "(SELECT oid FROM pg_namespace WHERE nspname = current_schema())";
+
 		static function connect(?string $server, string $username, string $password) {
 			$connection = parent::connect($server, $username, $password);
 			if (is_string($connection)) {
@@ -252,6 +255,10 @@ if (isset($_GET["pgsql"])) {
 			if (min_version(12, 0, $connection)) {
 				$this->generated = array("STORED");
 			}
+			$this->partitionBy = array("RANGE", "LIST");
+			if (!$connection->flavor) {
+				$this->partitionBy[] = "HASH";
+			}
 		}
 
 		function enumLength(array $field) {
@@ -324,6 +331,39 @@ if (isset($_GET["pgsql"])) {
 			}
 		}
 
+		function inheritsFrom(string $table): array {
+			return get_vals("SELECT relname FROM pg_class JOIN pg_inherits ON inhparent = oid WHERE inhrelid = " . $this->tableOid($table) . " ORDER BY 1");
+		}
+
+		function inheritedTables(string $table): array {
+			return get_vals("SELECT relname FROM pg_inherits JOIN pg_class ON inhrelid = oid WHERE inhparent = " . $this->tableOid($table) . " ORDER BY 1");
+		}
+
+		function partitionsInfo(string $table): array {
+			$row = connection()->query("SELECT * FROM pg_partitioned_table WHERE partrelid = " . driver()->tableOid($table))->fetch_assoc();
+			if ($row) {
+				$attrs = get_vals("SELECT attname FROM pg_attribute WHERE attrelid = $row[partrelid] AND attnum IN (" . str_replace(" ", ", ", $row["partattrs"]) . ")"); //! ordering
+				$by = array('h' => 'HASH', 'l' => 'LIST', 'r' => 'RANGE');
+				return array(
+					"partition_by" => $by[$row["partstrat"]],
+					"partition" => implode(", ", array_map('Adminer\idf_escape', $attrs)),
+				);
+			}
+			return array();
+		}
+
+		function tableOid(string $table): string {
+			return "(SELECT oid FROM pg_class WHERE relnamespace = $this->nsOid AND relname = " . q($table) . " AND relkind IN ('r', 'm', 'v', 'f', 'p'))";
+		}
+
+		function indexAlgorithms(array $tableStatus): array {
+			static $return = array();
+			if (!$return) {
+				$return = get_vals("SELECT amname FROM pg_am" . (min_version(9.6) ? " WHERE amtype = 'i'" : "") . " ORDER BY amname = 'btree' DESC, amname");
+			}
+			return $return;
+		}
+
 		function supportsIndex(array $table_status): bool {
 			// returns true for "materialized view"
 			return $table_status["Engine"] != "view";
@@ -406,18 +446,20 @@ ORDER BY 1";
 		$return = array();
 		foreach (
 			get_rows("SELECT
-	c.relname AS \"Name\",
-	CASE c.relkind WHEN 'r' THEN 'table' WHEN 'm' THEN 'materialized view' ELSE 'view' END AS \"Engine\"" . ($has_size ? ",
-	pg_table_size(c.oid) AS \"Data_length\",
-	pg_indexes_size(c.oid) AS \"Index_length\"" : "") . ",
-	obj_description(c.oid, 'pg_class') AS \"Comment\",
-	" . (min_version(12) ? "''" : "CASE WHEN c.relhasoids THEN 'oid' ELSE '' END") . " AS \"Oid\",
-	c.reltuples as \"Rows\",
-	n.nspname
-FROM pg_class c
-JOIN pg_namespace n ON(n.nspname = current_schema() AND n.oid = c.relnamespace)
+	relname AS \"Name\",
+	CASE relkind WHEN 'v' THEN 'view' WHEN 'm' THEN 'materialized view' ELSE 'table' END AS \"Engine\"" . ($has_size ? ",
+	pg_table_size(oid) AS \"Data_length\",
+	pg_indexes_size(oid) AS \"Index_length\"" : "") . ",
+	obj_description(oid, 'pg_class') AS \"Comment\",
+	" . (min_version(12) ? "''" : "CASE WHEN relhasoids THEN 'oid' ELSE '' END") . " AS \"Oid\",
+	reltuples as \"Rows\",
+	inhparent AS inherited,
+	current_schema() AS nspname
+FROM pg_class
+LEFT JOIN pg_inherits ON inhrelid = oid
 WHERE relkind IN ('r', 'm', 'v', 'f', 'p')
-" . ($name != "" ? "AND relname = " . q($name) : "ORDER BY relname")) as $row //! Index_length, Auto_increment
+AND relnamespace = " . driver()->nsOid . "
+" . ($name != "" ? "AND relname = " . q($name) : "ORDER BY relname")) as $row //! Auto_increment
 		) {
 			$return[$row["Name"]] = $row;
 		}
@@ -444,15 +486,12 @@ WHERE relkind IN ('r', 'm', 'v', 'f', 'p')
 	format_type(a.atttypid, a.atttypmod) AS full_type,
 	pg_get_expr(d.adbin, d.adrelid) AS default,
 	a.attnotnull::int,
-	col_description(c.oid, a.attnum) AS comment" . (min_version(10) ? ",
+	col_description(a.attrelid, a.attnum) AS comment" . (min_version(10) ? ",
 	a.attidentity" . (min_version(12) ? ",
 	a.attgenerated" : "") : "") . "
-FROM pg_class c
-JOIN pg_namespace n ON c.relnamespace = n.oid
-JOIN pg_attribute a ON c.oid = a.attrelid
-LEFT JOIN pg_attrdef d ON c.oid = d.adrelid AND a.attnum = d.adnum
-WHERE c.relname = " . q($table) . "
-AND n.nspname = current_schema()
+FROM pg_attribute a
+LEFT JOIN pg_attrdef d ON a.attrelid = d.adrelid AND a.attnum = d.adnum
+WHERE a.attrelid = " . driver()->tableOid($table) . "
 AND NOT a.attisdropped
 AND a.attnum > 0
 ORDER BY a.attnum") as $row
@@ -488,18 +527,22 @@ ORDER BY a.attnum") as $row
 	function indexes($table, $connection2 = null) {
 		$connection2 = connection($connection2);
 		$return = array();
-		$table_oid = get_val("SELECT oid FROM pg_class WHERE relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema()) AND relname = " . q($table), 0, $connection2);
+		$table_oid = driver()->tableOid($table);
 		$columns = get_key_vals("SELECT attnum, attname FROM pg_attribute WHERE attrelid = $table_oid AND attnum > 0", $connection2);
 		foreach (
-			get_rows("SELECT relname, indisunique::int, indisprimary::int, indkey, indoption, (indpred IS NOT NULL)::int as indispartial
-FROM pg_index i, pg_class ci
-WHERE i.indrelid = $table_oid AND ci.oid = i.indexrelid
+			get_rows("SELECT relname, indisunique::int, indisprimary::int, indkey, indoption, (indpred IS NOT NULL)::int as indispartial, pg_am.amname as algorithm, pg_get_expr(pg_index.indpred, pg_index.indrelid, true) AS partial
+FROM pg_index
+JOIN pg_class ON indexrelid = oid
+JOIN pg_am ON pg_am.oid = pg_class.relam
+WHERE indrelid = $table_oid
 ORDER BY indisprimary DESC, indisunique DESC", $connection2) as $row
 		) {
 			$relname = $row["relname"];
 			$return[$relname]["type"] = ($row["indispartial"] ? "INDEX" : ($row["indisprimary"] ? "PRIMARY" : ($row["indisunique"] ? "UNIQUE" : "INDEX")));
 			$return[$relname]["columns"] = array();
 			$return[$relname]["descs"] = array();
+			$return[$relname]["algorithm"] = $row["algorithm"];
+			$return[$relname]["partial"] = $row["partial"];
 			if ($row["indkey"]) {
 				foreach (explode(" ", $row["indkey"]) as $indkey) {
 					$return[$relname]["columns"][] = $columns[$indkey];
@@ -518,7 +561,7 @@ ORDER BY indisprimary DESC, indisunique DESC", $connection2) as $row
 		foreach (
 			get_rows("SELECT conname, condeferrable::int AS deferrable, pg_get_constraintdef(oid) AS definition
 FROM pg_constraint
-WHERE conrelid = (SELECT pc.oid FROM pg_class AS pc INNER JOIN pg_namespace AS pn ON (pn.oid = pc.relnamespace) WHERE pc.relname = " . q($table) . " AND pn.nspname = current_schema())
+WHERE conrelid = " . driver()->tableOid($table) . "
 AND contype = 'f'::char
 ORDER BY conkey, conname") as $row
 		) {
@@ -538,7 +581,7 @@ ORDER BY conkey, conname") as $row
 	}
 
 	function view($name) {
-		return array("select" => trim(get_val("SELECT pg_get_viewdef(" . get_val("SELECT oid FROM pg_class WHERE relnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema()) AND relname = " . q($name)) . ")")));
+		return array("select" => trim(get_val("SELECT pg_get_viewdef(" . driver()->tableOid($name) . ")")));
 	}
 
 	function collations() {
@@ -621,7 +664,31 @@ ORDER BY conkey, conname") as $row
 		}
 		$alter = array_merge($alter, $foreign);
 		if ($table == "") {
-			array_unshift($queries, "CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)");
+			$status = "";
+			if ($partitioning) {
+				$cockroach = (connection()->flavor == 'cockroach');
+				$status = " PARTITION BY $partitioning[partition_by]($partitioning[partition])";
+				if ($partitioning["partition_by"] == 'HASH') {
+					$partitions = +$partitioning["partitions"];
+					for ($i=0; $i < $partitions; $i++) {
+						$queries[] = "CREATE TABLE " . idf_escape($name . "_$i") . " PARTITION OF " . idf_escape($name) . " FOR VALUES WITH (MODULUS $partitions, REMAINDER $i)";
+					}
+				} else {
+					$prev = "MINVALUE";
+					foreach ($partitioning["partition_names"] as $i => $val) {
+						$value = $partitioning["partition_values"][$i];
+						$partition = " VALUES " . ($partitioning["partition_by"] == 'LIST' ? "IN ($value)" : "FROM ($prev) TO ($value)");
+						if ($cockroach) {
+							$status .= ($i ? "," : " (") . "\n  PARTITION " . (preg_match('~^DEFAULT$~i', $val) ? $val : idf_escape($val)) . "$partition";
+						} else {
+							$queries[] = "CREATE TABLE " . idf_escape($name . "_$val") . " PARTITION OF " . idf_escape($name) . " FOR$partition";
+						}
+						$prev = $value;
+					}
+					$status .= ($cockroach ? "\n)" : "");
+				}
+			}
+			array_unshift($queries, "CREATE TABLE " . table($name) . " (\n" . implode(",\n", $alter) . "\n)$status");
 		} elseif ($alter) {
 			array_unshift($queries, "ALTER TABLE " . table($table) . "\n" . implode(",\n", $alter));
 		}
@@ -648,7 +715,7 @@ ORDER BY conkey, conname") as $row
 		$queries = array();
 		foreach ($alter as $val) {
 			if ($val[0] != "INDEX") {
-				//! descending UNIQUE indexes results in syntax error
+				//! descending UNIQUE indexes result in syntax error
 				$create[] = ($val[2] == "DROP"
 					? "\nDROP CONSTRAINT " . idf_escape($val[1])
 					: "\nADD" . ($val[1] != "" ? " CONSTRAINT " . idf_escape($val[1]) : "") . " $val[0] " . ($val[0] == "PRIMARY" ? "KEY " : "") . "(" . implode(", ", $val[2]) . ")"
@@ -656,7 +723,12 @@ ORDER BY conkey, conname") as $row
 			} elseif ($val[2] == "DROP") {
 				$drop[] = idf_escape($val[1]);
 			} else {
-				$queries[] = "CREATE INDEX " . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_")) . " ON " . table($table) . " (" . implode(", ", $val[2]) . ")";
+				$queries[] = "CREATE INDEX " . idf_escape($val[1] != "" ? $val[1] : uniqid($table . "_"))
+					. " ON " . table($table)
+					. ($val[3] ? " USING $val[3]" : "")
+					. " (" . implode(", ", $val[2]) . ")"
+					. ($val[4] ? " WHERE $val[4]" : "")
+				;
 			}
 		}
 		if ($create) {
@@ -798,7 +870,7 @@ ORDER BY SPECIFIC_NAME');
 		return get_key_vals(
 			"SELECT oid, typname
 FROM pg_type
-WHERE typnamespace = (SELECT oid FROM pg_namespace WHERE nspname = current_schema())
+WHERE typnamespace = " . driver()->nsOid . "
 AND typtype IN ('b','d','e')
 AND typelem = 0"
 		);
@@ -899,8 +971,16 @@ AND typelem = 0"
 		foreach (driver()->checkConstraints($table) as $conname => $consrc) {
 			$return_parts[] = "CONSTRAINT " . idf_escape($conname) . " CHECK $consrc";
 		}
+		$return .= implode(",\n    ", $return_parts) . "\n)";
 
-		$return .= implode(",\n    ", $return_parts) . "\n) WITH (oids = " . ($status['Oid'] ? 'true' : 'false') . ");";
+		$partition = driver()->partitionsInfo($status['Name']);
+		if ($partition) {
+			$return .= "\nPARTITION BY $partition[partition_by]($partition[partition])";
+		}
+		//! parse pg_class.relpartbound to create PARTITION OF
+		//! don't insert partitioned data twice
+
+		$return .= "\nWITH (oids = " . ($status['Oid'] ? 'true' : 'false') . ");";
 
 		// comments for table & fields
 		if ($status['Comment']) {
@@ -955,9 +1035,11 @@ AND typelem = 0"
 	}
 
 	function support($feature) {
-		return preg_match('~^(check|database|table|columns|sql|indexes|descidx|comment|view|' . (min_version(9.3) ? 'materializedview|' : '') . 'scheme|' . (min_version(11) ? 'procedure|' : '') . 'routine|sequence|trigger|type|variables|drop_col'
+		return preg_match('~^(check|columns|comment|database|drop_col|dump|descidx|indexes|kill|partial_indexes|routine|scheme|sequence|sql|table|trigger|type|variables|view'
+			. (min_version(9.3) ? '|materializedview' : '')
+			. (min_version(11) ? '|procedure' : '')
 			. (connection()->flavor == 'cockroach' ? '' : '|processlist') // https://github.com/cockroachdb/cockroach/issues/24745
-			. '|kill|dump)$~', $feature)
+			. ')$~', $feature)
 		;
 	}
 
