@@ -108,15 +108,24 @@ class Adminer {
 		return true;
 	}
 
+	/** Print extra classes in <body class>; must start with a space */
+	function bodyClass(): void {
+		echo " adminer";
+	}
+
 	/** Get URLs of the CSS files
-	* @return list<string>
+	* @return string[] key is URL, value is either 'light' (supports only light color scheme), 'dark' or '' (both)
 	*/
 	function css(): array {
 		$return = array();
 		foreach (array("", "-dark") as $mode) {
 			$filename = "adminer$mode.css";
 			if (file_exists($filename)) {
-				$return[] = "$filename?v=" . crc32(file_get_contents($filename));
+				$file = file_get_contents($filename);
+				$return["$filename?v=" . crc32($file)] = ($mode
+					? "dark"
+					: (preg_match('~prefers-color-scheme:\s*dark~', $file) ? '' : 'light')
+				);
 			}
 		}
 		return $return;
@@ -179,6 +188,7 @@ class Adminer {
 	* @param ?string $set new item options, NULL for no new item
 	*/
 	function selectLinks(array $tableStatus, ?string $set = ""): void {
+		$name = $tableStatus["Name"];
 		echo '<p class="links">';
 		$links = array("select" => lang('Select data'));
 		if (support("table") || support("indexes")) {
@@ -196,7 +206,6 @@ class Adminer {
 		if ($set !== null) {
 			$links["edit"] = lang('New item');
 		}
-		$name = $tableStatus["Name"];
 		foreach ($links as $key => $val) {
 			echo " <a href='" . h(ME) . "$key=" . urlencode($name) . ($key == "edit" ? $set : "") . "'" . bold(isset($_GET[$key])) . ">$val</a>";
 		}
@@ -343,9 +352,15 @@ class Adminer {
 
 	/** Print list of indexes on table in tabular format
 	* @param Index[] $indexes
+	* @param TableStatus $tableStatus
 	*/
-	function tableIndexesPrint(array $indexes): void {
+	function tableIndexesPrint(array $indexes, array $tableStatus): void {
+		$partial = false;
+		foreach ($indexes as $name => $index) {
+			$partial |= !!$index["partial"];
+		}
 		echo "<table>\n";
+		$default_algorithm = first(driver()->indexAlgorithms($tableStatus));
 		foreach ($indexes as $name => $index) {
 			ksort($index["columns"]); // enforce correct columns order
 			$print = array();
@@ -355,7 +370,14 @@ class Adminer {
 					. ($index["descs"][$key] ? " DESC" : "")
 				;
 			}
-			echo "<tr title='" . h($name) . "'><th>$index[type]<td>" . implode(", ", $print) . "\n";
+
+			echo "<tr title='" . h($name) . "'>";
+			echo "<th>$index[type]" . ($default_algorithm && $index['algorithm'] != $default_algorithm ? " ($index[algorithm])" : "");
+			echo "<td>" . implode(", ", $print);
+			if ($partial) {
+				echo "<td>" . ($index['partial'] ? "<code class='jush-" . JUSH . "'>WHERE " . h($index['partial']) : "");
+			}
+			echo "\n";
 		}
 		echo "</table>\n";
 	}
@@ -395,7 +417,7 @@ class Adminer {
 		foreach ($indexes as $i => $index) {
 			if ($index["type"] == "FULLTEXT") {
 				echo "<div>(<i>" . implode("</i>, <i>", array_map('Adminer\h', $index["columns"])) . "</i>) AGAINST";
-				echo " <input type='search' name='fulltext[$i]' value='" . h($_GET["fulltext"][$i]) . "'>";
+				echo " <input type='search' name='fulltext[$i]' value='" . h(idx($_GET["fulltext"], $i)) . "'>";
 				echo script("qsl('input').oninput = selectFieldChange;", "");
 				echo checkbox("boolean[$i]", 1, isset($_GET["boolean"][$i]), "BOOL");
 				echo "</div>\n";
@@ -533,7 +555,7 @@ class Adminer {
 	function selectSearchProcess(array $fields, array $indexes): array {
 		$return = array();
 		foreach ($indexes as $i => $index) {
-			if ($index["type"] == "FULLTEXT" && $_GET["fulltext"][$i] != "") {
+			if ($index["type"] == "FULLTEXT" && idx($_GET["fulltext"], $i) != "") {
 				$return[] = "MATCH (" . implode(", ", array_map('Adminer\idf_escape', $index["columns"])) . ") AGAINST (" . q($_GET["fulltext"][$i]) . (isset($_GET["boolean"][$i]) ? " IN BOOLEAN MODE" : "") . ")";
 			}
 		}
@@ -1078,7 +1100,7 @@ class Adminer {
 		foreach ($tables as $table => $status) {
 			$table = "$table"; // do not highlight "0" as active everywhere
 			$name = adminer()->tableName($status);
-			if ($name != "") {
+			if ($name != "" && !$status["inherited"]) {
 				echo '<li><a href="' . h(ME) . 'select=' . urlencode($table) . '"'
 					. bold($_GET["select"] == $table || $_GET["edit"] == $table, "select")
 					. " title='" . lang('Select data') . "'>" . lang('select') . "</a> "
